// GMDLL_TesterDlg.cpp : implementation file
//

#include "stdafx.h"
#include "GMDLL_Tester.h"
#include "GMDLL_TesterDlg.h"

#include "GlobalMapperInterface_3D.h"
#include "GlobalMapperInterface_GPS.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(CGMDLL_TesterDlg, CDialog)
	//{{AFX_MSG_MAP(CGMDLL_TesterDlg)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_CLOSE_LAYER, OnCloseLayer)
	ON_BN_CLICKED(IDC_LOAD_LAYER, OnLoadLayer)
	ON_BN_CLICKED(IDC_SAVE_AS_GEOTIFF, OnSaveAsGeotiff)
	ON_BN_CLICKED(IDC_SAVE_AS_JPG, OnSaveAsJpg)
	ON_BN_CLICKED(IDC_TOGGLE_AUTO_CROP, OnToggleAutoCrop)
	ON_BN_CLICKED(IDC_PAN_DOWN, OnPanDown)
	ON_BN_CLICKED(IDC_PAN_LEFT, OnPanLeft)
	ON_BN_CLICKED(IDC_PAN_RIGHT, OnPanRight)
	ON_BN_CLICKED(IDC_PAN_UP, OnPanUp)
	ON_BN_CLICKED(IDC_ZOOM_IN, OnZoomIn)
	ON_BN_CLICKED(IDC_ZOOM_OUT, OnZoomOut)
	ON_BN_CLICKED(IDC_ZOOM_TO_ALL, OnZoomToAll)
	ON_BN_CLICKED(IDC_ZOOM_TO_SEL_LAYER, OnZoomToSelLayer)
	ON_WM_MOUSEMOVE()
	ON_BN_CLICKED(IDC_LOAD_TERRASERVER, OnLoadTerraserver)
	ON_BN_CLICKED(IDC_DRAW_GRID_LINES, OnDrawGridLines)
	ON_BN_CLICKED(IDC_GRID_GLOBAL, OnGridGlobal)
	ON_BN_CLICKED(IDC_GRID_LAT_LON, OnGridLatLon)
	ON_WM_LBUTTONUP()
	ON_BN_CLICKED(IDC_SET_PROJ_TO_GEOGRAPHIC, OnSetProjToGeographic)
	ON_BN_CLICKED(IDC_3D_VIEW, On3dView)
	ON_WM_RBUTTONUP()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
    ON_BN_CLICKED(IDC_LAYER_DISPLAY_OPTIONS, &CGMDLL_TesterDlg::OnBnClickedLayerDisplayOptions)
END_MESSAGE_MAP()

//===================================================================
//
// FUNCTIONS:
//   GPSCallbackFunc
//
// DESCRIPTION:
//   Callback function for handling GPS events. These messages come
//   in on a background thread, so keep that in mind when doing any
//   user interface handling.
//
//===================================================================
static void 
_stdcall GPSCallbackFunc
    ( 
    GM_gps_event_t32    aEvent,
    void*               aEventData
    )
{
    switch ( aEvent )
    {
        case GM_GPS_EVENT_NEW_POS:       // a new location is available (event data is GM_Point_t* with current lat/lon)
        {
            // The event data is the new position (in lat/lon/WGS84 degrees)
            const GM_Point_t* theNewPos = (const GM_Point_t*)aEventData;

            // Get the altitude in meters
            float theAlt = 0.0;
            GM_GPSGetAltitude( &theAlt );
            break;
        }

        case GM_GPS_EVENT_STATE_CHANGED: // the GPS state has changed
            break;

        case GM_GPS_EVENT_END_OF_FILE:   // the end of a NMEA data file being replayed has been reached
            break;
    }
} // end of function GPSCallbackFunc

//===================================================================
//
// FUNCTIONS:
//   MessageCallbackFunc
//
// DESCRIPTION:
//   Callback function for displaying messages. This doesn't do
//   anything different than the SDK would do by itself, it's just
//   to illustrate how the callback is used.
//
//===================================================================
static void
__stdcall MessageCallbackFunc
    (
    const char* aStr
    )
{
    // Just display the message
    ::AfxMessageBox( aStr );
} // end of function MessageCallbackFunc

//===================================================================
//
// FUNCTIONS:
//   ProgressCallbackFunc
//
// DESCRIPTION:
//   Callback function for handling progress display. You would
//   implement this and pass it to the SDK via the 
//   GM_SetProgressCallback function if you wanted to handle
//   progress in some non-default way. This should return TRUE
//   unless you want to abort the operation that the progress is
//   being reported on.
//
//===================================================================
static boolean 
_stdcall ProgressCallbackFunc
    ( 
    uint32                  aOperationIdx,  // index of operation this progress applies to (useful if nested long operations or multiple threads)
    GM_ProgressUpdate_t8    aUpdateType,    // what kind of progress update is it
    const char*             aStatus,        // current status string
    uint32                  aPercent        // percent complete
    )
{
    // No progress display here, just return TRUE to keep going
    return ( TRUE );
} // end of function ProgressCallbackFunc

/////////////////////////////////////////////////////////////////////////////
// CGMDLL_TesterDlg dialog

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::CGMDLL_TesterDlg
//
// DESCRIPTION:
//   Constructor for CGMDLL_TesterDlg class.
//
//===================================================================
CGMDLL_TesterDlg::CGMDLL_TesterDlg
    (
    CWnd*   aParent
    )
: CDialog( CGMDLL_TesterDlg::IDD, aParent )
{
	//{{AFX_DATA_INIT(CGMDLL_TesterDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT

	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	mIcon = AfxGetApp()->LoadIcon( IDR_MAINFRAME );
} // end of function CGMDLL_TesterDlg::CGMDLL_TesterDlg

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::~CGMDLL_TesterDlg
//
// DESCRIPTION:
//   Destructor for CGMDLL_TesterDlg class.
//
//===================================================================
CGMDLL_TesterDlg::~CGMDLL_TesterDlg()
{
} // end of function CGMDLL_TesterDlg::~CGMDLL_TesterDlg

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::DisplayVectorFeatureInfo
//
// DESCRIPTION:
//   Displays an information dialog about a particular vector feature.
//
//===================================================================
void
CGMDLL_TesterDlg::DisplayVectorFeatureInfo
    (
    GM_FeatureClassType_t8      aFeatureClass,
    const GM_VectorFeature_t&   aFeature
    )
{
    // Get information about the classification of the feature
    GM_FeatureClassInfo_t theClassInfo;
    ::memset( &theClassInfo, 0, sizeof theClassInfo );
    GM_GetFeatureClassInfo( aFeature.mClass, aFeatureClass, &theClassInfo );

    // Build an information string about the feature
    CString theMsgStr;
    theMsgStr.Format
        (
        _T("Found Feature\n")
        _T("LABEL: <%s>\n")
        _T("DESC: <%s>\n")
        _T("TYPE: <%s>"),
        ( NULL == aFeature.mName ) ? _T("") : aFeature.mName,
        ( NULL == aFeature.mDesc ) ? _T("") : aFeature.mDesc,
        ( NULL == theClassInfo.mDesc ) ? _T("???") : theClassInfo.mDesc
        );

    // Add the attribute values
    for ( uint32 i = 0; i < aFeature.mNumAttrs; i++ )
    {
        // Add a header for the first attribute
        if ( 0 == i )
        {
            theMsgStr += _T("\n\nATTRIBUTE/VALUE LIST:\n");
        }
        
        // Add the attribute value
        theMsgStr += CString( aFeature.mAttrList[i].mName );
        theMsgStr += _T("=");
        theMsgStr += CString( aFeature.mAttrList[i].mVal );
        theMsgStr += _T("\n");
    }

    // Display the dialog
    ::AfxMessageBox( theMsgStr );
} // end of function CGMDLL_TesterDlg::DisplayVectorFeatureInfo

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::DoDataExchange
//
// DESCRIPTION:
//   Performs DDX/DDV.
//
//===================================================================
void 
CGMDLL_TesterDlg::DoDataExchange
    (
    CDataExchange* pDX
    )
{
	CDialog::DoDataExchange( pDX );

	//{{AFX_DATA_MAP(CGMDLL_TesterDlg)
	DDX_Control(pDX, IDC_LAYER_LIST, mLayerList);
	//}}AFX_DATA_MAP
} // end of function CGMDLL_TesterDlg::DoDataExchange

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::FillLayerList
//
// DESCRIPTION:
//   Fills the list of loaded layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::FillLayerList() 
{
    // Add a column if we haven't done that yet
    if ( 0 == mLayerList.GetHeaderCtrl()->GetItemCount() )
    {
        mLayerList.InsertColumn( 0, "Description" );
    }

    // Clear the layer list
    mLayerList.DeleteAllItems();

    // Add an item for each layer
    const GM_MapWindow::LayerHandleList_t& theLayerHandleList = 
        mMapWindow.GetLayerList();
    for ( uint32 i = 0; i < theLayerHandleList.size(); i++ )
    {
        GM_LayerHandle_t32 theLayer = theLayerHandleList[i];

        // Get info about the layer
        const GM_LayerInfo_t* theLayerInfo = GM_GetLayerInfo( theLayer );
        if ( NULL == theLayerInfo )
        {
            ::AfxMessageBox( "Error getting layer info" );
            continue;
        }

        // Add the description to the list
        sint32 theIdx = mLayerList.InsertItem
            ( 
            mLayerList.GetItemCount(), theLayerInfo->mDescription 
            );
        mLayerList.SetItemData( theIdx, (DWORD)theLayer );
    }

    // Size the column to fill the list
    CRect theListRect;
    mLayerList.GetClientRect( theListRect );
    mLayerList.SetColumnWidth( 0, theListRect.Width() );
} // end of function CGMDLL_TesterDlg::FillLayerList

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::On3dView
//
// DESCRIPTION:
//   Called when the 3D view should be shown or hidden.
//
//===================================================================
void 
CGMDLL_TesterDlg::On3dView() 
{
    // If the 3D view is already shown, close it. Otherwise show it.
    if ( GM_3DIsWindowOpen() )
    {
        mMapWindow.Hide3DView();
    }
    else
    {
        mMapWindow.Show3DView();
    }
} // end of function CGMDLL_TesterDlg::On3dView

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnBnClickedLayerDisplayOptions
//
// DESCRIPTION:
//   Shows display options dialog for selected layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnBnClickedLayerDisplayOptions()
{
    // Build the list of selected layers
    std::vector<GM_LayerHandle_t32> theSelLayers;
    sint32 theSel = mLayerList.GetNextItem( -1, LVNI_SELECTED );
    if ( -1 == theSel )
    {
        ::AfxMessageBox( "You must select one or more layer(s)" );
        return;
    }
    while ( -1 != theSel )
    {
        GM_LayerHandle_t32 theLayer = 
            (GM_LayerHandle_t32)mLayerList.GetItemData( theSel );        ;
        theSelLayers.push_back( theLayer );

        // Get the next selected item
        theSel = mLayerList.GetNextItem( theSel, LVNI_SELECTED );
    }

    // Show the options dialog
    mMapWindow.ShowLayerDisplayOptionsDialog( theSelLayers );
} // end of function CGMDLL_TesterDlg::OnBnClickedLayerDisplayOptions

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnCloseLayer
//
// DESCRIPTION:
//   Closes the selected layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnCloseLayer() 
{
    // Close each selected layer
    sint32 theSel = mLayerList.GetNextItem( -1, LVNI_SELECTED );
    if ( -1 == theSel )
    {
        ::AfxMessageBox( "You must select the layer(s) to close" );
        return;
    }
    while ( -1 != theSel )
    {
        // Remove this layer
        GM_LayerHandle_t32 theLayer = 
            (GM_LayerHandle_t32)mLayerList.GetItemData( theSel );        ;
        if ( GM_Error_None != mMapWindow.UnloadLayer( theLayer ) )
        {
            ::AfxMessageBox( "Error closing layer" );
        }

        // Get the next selected item
        theSel = mLayerList.GetNextItem( theSel, LVNI_SELECTED );
    }

    // Refill the layer list
    FillLayerList();

    // Zoom out to all layers
    mMapWindow.ZoomToAllLayers();
} // end of function CGMDLL_TesterDlg::OnCloseLayer

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnDrawGridLines
//
// DESCRIPTION:
//   Called when the grid line display is toggled.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnDrawGridLines() 
{
    // Update the grid display type
    if ( IsDlgButtonChecked( IDC_DRAW_GRID_LINES ) )
    {
        if ( IsDlgButtonChecked( IDC_GRID_LAT_LON ) )
        {
            mMapWindow.SetGridDisplayType( GM_MapWindow::eLatLonGrid );
        }
        else
        {
            mMapWindow.SetGridDisplayType( GM_MapWindow::eViewProjGrid );
        }
    }
    else
    {
        mMapWindow.SetGridDisplayType( GM_MapWindow::eNoGrid );
    }
} // end of function CGMDLL_TesterDlg::OnDrawGridLines

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnEraseBkgnd
//
// DESCRIPTION:
//   Called when the background of the dialog needs erased. This is
//   overridden so that we can prevent the map area from being erased
//   in order to ensure flicker-free operation.
//
//===================================================================
BOOL 
CGMDLL_TesterDlg::OnEraseBkgnd
    (
    CDC* pDC
    ) 
{
    // Get the map area window and rectangle
    CWnd* theMapWnd = GetDlgItem( IDC_DRAW_WINDOW );
    RECT theMapWndRect;
    theMapWnd->GetWindowRect( &theMapWndRect );
    ScreenToClient( &theMapWndRect );

    // Setup a clip region to exclude the draw area to avoid flicker
    // of the map drawing region
    CRgn theOrigRgn;
    int theRgnType = GetUpdateRgn( &theOrigRgn, FALSE );
    pDC->ExcludeClipRect( &theMapWndRect );

    // Call the base class implementation
    BOOL theRetVal = CDialog::OnEraseBkgnd( pDC );

    // Restore the old clip region
    pDC->SelectClipRgn( &theOrigRgn );

    return ( theRetVal );
} // end of function CGMDLL_TesterDlg::OnEraseBkgnd

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnGridGlobal
//
// DESCRIPTION:
//   Called when the global projection grid display option is
//   selected.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnGridGlobal() 
{
    // Redraw if we're drawing grid lines
    if ( IsDlgButtonChecked( IDC_DRAW_GRID_LINES ) )
    {
        mMapWindow.SetGridDisplayType( GM_MapWindow::eViewProjGrid );
    }
} // end of function CGMDLL_TesterDlg::OnGridGlobal

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnGridLatLon
//
// DESCRIPTION:
//   Called when the lat/lon grid display option is selected.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnGridLatLon() 
{
    // Redraw if we're drawing grid lines
    if ( IsDlgButtonChecked( IDC_DRAW_GRID_LINES ) )
    {
        mMapWindow.SetGridDisplayType( GM_MapWindow::eLatLonGrid );
    }
} // end of function CGMDLL_TesterDlg::OnGridLatLon

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnInitDialog
//
// DESCRIPTION:
//   Performs dialog initialization.
//
//===================================================================
BOOL 
CGMDLL_TesterDlg::OnInitDialog()
{
    // Call the base class implementation
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon( mIcon, TRUE );	// Set big icon
	SetIcon( mIcon, FALSE );  // Set small icon

    // Get the map area window and rectangle
    CWnd* theMapWnd = GetDlgItem( IDC_DRAW_WINDOW );
    RECT theMapWndRect;
    theMapWnd->GetWindowRect( &theMapWndRect );
    ScreenToClient( &theMapWndRect );

    // Setup the window title to display the SDK version and build date
    uint32 theSDKVer = GM_GetSDKVersion();
    CTime theBuildDate( GM_GetSDKBuildDate() );
    CString theOldTitleStr;
    GetWindowText( theOldTitleStr );
    CString theTitleStr;
    theTitleStr.Format
        (
        _T("%s - v%d.%02d (SDK Build Date: %s)"), 
        theOldTitleStr, 
        theSDKVer / 100, 
        theSDKVer % 100,
        theBuildDate.Format( _T("%x") )
        );
    SetWindowText( theTitleStr );

    // Setup the map window to draw to the dialog window in the area of the
    // IDC_MAP_AREA control
    mMapWindow.SetDrawWindow( GetSafeHwnd() );
    mMapWindow.SetDrawWindowRect( theMapWndRect );

    // Initially check the global projection grid
    CheckDlgButton( IDC_GRID_GLOBAL, 1 );

    // Set the Global Mapper SDK DLL to save/restore its settings from
    // the GMDLL_Tester registry setting rather than the default one shared
    // with the Global Mapper application.
    GM_SetRegistryKey( "GMDLL_Tester" );

    // Set the error callback function so that we can handle any warning
    // or error messages ourselves.
    GM_SetMessageCallback( MessageCallbackFunc );

	return ( TRUE );
} // end of function CGMDLL_TesterDlg::OnInitDialog

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnLoadLayer
//
// DESCRIPTION:
//   Loads a new layer.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnLoadLayer() 
{
    if ( GM_Error_None == mMapWindow.LoadLayer() )
    {
        // Refill our layer list
        FillLayerList();
    }
} // end of function CGMDLL_TesterDlg::OnLoadLayer

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnLoadTerraserver
//
// DESCRIPTION:
//   Loads data from an online data source.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnLoadTerraserver() 
{
    if ( GM_Error_None == mMapWindow.LoadOnlineLayer() )
    {
        // Refill our layer list
        FillLayerList();
    }
} // end of function CGMDLL_TesterDlg::OnLoadTerraserver

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnLButtonUp
//
// DESCRIPTION:
//   Called when the left mouse button is released. If the button
//   was clicked in the map area, the map is recentered on that
//   point.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnLButtonUp
    (
    UINT    nFlags, 
    CPoint  point
    ) 
{
    // If the point was in the map window, recenter on the point
    if ( ::PtInRect( &mMapWindow.GetDrawWndRect(), point ) )
    {
        mMapWindow.CenterOnPixel( point.x, point.y );
    }

    // Call the base class implementation
	CDialog::OnLButtonUp( nFlags, point );
} // end of function CGMDLL_TesterDlg::OnLButtonUp

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnMouseMove
//
// DESCRIPTION:
//   Handles updated the cursor coordinate display when the mouse
//   is moved.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnMouseMove
    (
    UINT    nFlags, 
    CPoint  point
    ) 
{
    // If the point is in the draw window and we have some layers loaded, 
    // determine the global and lat/lon bounds of the point
    CString theGlobalStr = "No Data Loaded";
    CString theLatLonStr = "No Data Loaded";
    BOOL isPtInRect = ::PtInRect( &mMapWindow.GetDrawWndRect(), point );
    if ( isPtInRect && !mMapWindow.GetLayerList().empty() )
    {
        // Convert the coordinate from screen coordinate to ground coordinates
        double theGroundX, theGroundY;
        GM_Error_t32 theErr = mMapWindow.ConvertCoordPixelToGround
            (
            point.x, point.y, theGroundX, theGroundY
            );
        if ( GM_Error_None == theErr )
        {
            theGlobalStr.Format( _T("%.6f, %.6f"), theGroundX, theGroundY );
        }
        else
        {
            theGlobalStr.Format( _T("Error %d"), theErr );
        }

        // Get the lat/lon coordinates
        if ( GM_Error_None == theErr )
        {
            // Project the pixel coordinates to lat/lon
            double theLat, theLon;
            theErr = mMapWindow.ConvertCoordPixelToLatLon
                ( 
                point.x, point.y, theLat, theLon
                );
            if ( GM_Error_None == theErr )
            {
                theLatLonStr.Format( _T("%.6f, %.6f"), theLat, theLon );
            }
            else
            {
                theLatLonStr.Format( _T("GM_ProjectPoint error %d"), theErr );
            }
        }
        else
        {
            theLatLonStr = _T("Unable to get ground coordinates");
        }
    }
    else if ( !isPtInRect )
    {
        theGlobalStr = theLatLonStr = _T("Cursor Not in Draw Window");
    }

    // Update the text of the controls
    GetDlgItem( IDC_CUR_POS_GROUND )->SetWindowText( theGlobalStr );
    GetDlgItem( IDC_CUR_POS_LAT_LON )->SetWindowText( theLatLonStr );

    // Call the base class implementation
	CDialog::OnMouseMove( nFlags, point );
} // end of function CGMDLL_TesterDlg::OnMouseMove

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnPaint
//
// DESCRIPTION:
//   Handles rendering the dialog contents.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnPaint() 
{
    // Create the device context for painting
    CPaintDC thePaintDC( this ); // device context for painting

    // Make sure we don't re-enter this function
    static bool isDrawing = false;
    if ( isDrawing )
    {
        return;
    }
    isDrawing = true;

    // Draw the dialog
	if ( IsIconic() )
	{
		SendMessage( WM_ICONERASEBKGND, (WPARAM)thePaintDC.GetSafeHdc(), 0 );

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics( SM_CXICON );
		int cyIcon = GetSystemMetrics( SM_CYICON );
		CRect theClientRect;
		GetClientRect( &theClientRect );
		int x = ( theClientRect.Width() - cxIcon + 1 ) / 2;
		int y = ( theClientRect.Height() - cyIcon + 1 ) / 2;

		// Draw the icon
		thePaintDC.DrawIcon( x, y, mIcon );
	}
	else
	{
        // Show a wait cursor
        CWaitCursor theWC;

        // Draw the layers
        mMapWindow.Draw
            ( 
            thePaintDC.GetSafeHdc(), 
            RGB(255,255,206),
            GM_MapWindow::eDrawOrderDefault,
            true,
            GM_DrawFlags_IncrementalRender
            );
	}

    // Clear the drawing flag
    isDrawing = false;
} // end of function CGMDLL_TesterDlg::OnPaint

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnPanDown
//
// DESCRIPTION:
//   Pans the display down a quarter of a screen.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnPanDown() 
{
    // Get the current draw rectangle
    RECT theDrawRect = mMapWindow.GetDrawWndRect();

    // Get the center of the draw rectangle
    sint32 theCenX = ( theDrawRect.left + theDrawRect.right ) / 2;
    sint32 theCenY = ( theDrawRect.top + theDrawRect.bottom ) / 2;

    // Shift the y coordinates of the center up by one quarter of the height
    theCenY += ( theDrawRect.bottom - theDrawRect.top ) / 4;

    // Recenter on the new center
    mMapWindow.CenterOnPixel( theCenX, theCenY );
} // end of function CGMDLLTesterDlg::OnPanDown

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnPanLeft
//
// DESCRIPTION:
//   Pans the display left a quarter of a screen.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnPanLeft() 
{
    // Get the current draw rectangle
    RECT theDrawRect = mMapWindow.GetDrawWndRect();

    // Get the center of the draw rectangle
    sint32 theCenX = ( theDrawRect.left + theDrawRect.right ) / 2;
    sint32 theCenY = ( theDrawRect.top + theDrawRect.bottom ) / 2;

    // Shift the x coordinates of the center left by one quarter of the width
    theCenX -= ( theDrawRect.right - theDrawRect.left ) / 4;

    // Recenter on the new center
    mMapWindow.CenterOnPixel( theCenX, theCenY );
} // end of function CGMDLLTesterDlg::OnPanLeft

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnPanRight
//
// DESCRIPTION:
//   Pans the display right a quarter of a screen.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnPanRight() 
{
    // Get the current draw rectangle
    RECT theDrawRect = mMapWindow.GetDrawWndRect();

    // Get the center of the draw rectangle
    sint32 theCenX = ( theDrawRect.left + theDrawRect.right ) / 2;
    sint32 theCenY = ( theDrawRect.top + theDrawRect.bottom ) / 2;

    // Shift the x coordinates of the center right by one quarter of the width
    theCenX += ( theDrawRect.right - theDrawRect.left ) / 4;

    // Recenter on the new center
    mMapWindow.CenterOnPixel( theCenX, theCenY );
} // end of function CGMDLLTesterDlg::OnPanRight

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnPanUp
//
// DESCRIPTION:
//   Pans the display up a quarter of a screen.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnPanUp() 
{
    // Get the current draw rectangle
    RECT theDrawRect = mMapWindow.GetDrawWndRect();

    // Get the center of the draw rectangle
    sint32 theCenX = ( theDrawRect.left + theDrawRect.right ) / 2;
    sint32 theCenY = ( theDrawRect.top + theDrawRect.bottom ) / 2;

    // Shift the y coordinates of the center down by one quarter of the height
    theCenY -= ( theDrawRect.bottom - theDrawRect.top ) / 4;

    // Recenter on the new center
    mMapWindow.CenterOnPixel( theCenX, theCenY );
} // end of function CGMDLLTesterDlg::OnPanUp

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnQueryDragIcon
//
// DESCRIPTION:
//   Returns the icon to display when dragging the application.
//
//===================================================================
HCURSOR 
CGMDLL_TesterDlg::OnQueryDragIcon()
{
	return ( (HCURSOR)mIcon );
} // end of function CGMDLL_TesterDlg::OnQueryDragIcon

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnRButtonUp
//
// DESCRIPTION:
//   Called when the right mouse button is released. If the button
//   was clicked in the map area, a nearest search is performed to
//   find the nearest vector feature to the click location and
//   information about that feature is displayed.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnRButtonUp
    (
    UINT    nFlags, 
    CPoint  point
    ) 
{
    // If the point was in the map window, recenter on the point
    if ( ::PtInRect( &mMapWindow.GetDrawWndRect(), point ) )
    {
        // Find the nearest feature
        GM_MapWindow::FoundFeatureList_t theFindResults =
            mMapWindow.FindNearestFeatures( point, 20 );

        // Display the results
        if ( !theFindResults.empty() )
        {
            // Display information about the feature based on what type
            // it is.            
            if ( GM_FeatureClass_Area == theFindResults.front().mFeatureClassType )
            {
                // Get the area feature
                GM_AreaFeature_t* theArea = GM_GetAreaFeature
                    (
                    theFindResults.front().mLayer, theFindResults.front().mFeatureIndex
                    );
                if ( NULL == theArea )
                {
                    ::AfxMessageBox( _T("Unable to get info for nearest area.") );
                }
                else
                {
                    // Display an info dialog
                    DisplayVectorFeatureInfo
                        ( 
                        theFindResults.front().mFeatureClassType, theArea->mFeatureInfo 
                        );

                    // Free the area feature
                    GM_FreeAreaFeature( theArea );
                    theArea = NULL;
                }
            }
            else if ( GM_FeatureClass_Line == theFindResults.front().mFeatureClassType )
            {
                // Get the line feature
                GM_LineFeature_t* theLine = GM_GetLineFeature
                    (
                    theFindResults.front().mLayer, theFindResults.front().mFeatureIndex
                    );
                if ( NULL == theLine )
                {
                    ::AfxMessageBox( _T("Unable to get info for nearest line.") );
                }
                else
                {
                    // Display an info dialog
                    DisplayVectorFeatureInfo
                        ( 
                        theFindResults.front().mFeatureClassType, theLine->mFeatureInfo 
                        );

                    // Free the line feature
                    GM_FreeLineFeature( theLine );
                    theLine = NULL;
                }
            }
            else if ( GM_FeatureClass_Point == theFindResults.front().mFeatureClassType )
            {
                // Get the point feature
                GM_PointFeature_t* thePoint = GM_GetPointFeature
                    (
                    theFindResults.front().mLayer, theFindResults.front().mFeatureIndex
                    );
                if ( NULL == thePoint )
                {
                    ::AfxMessageBox( _T("Unable to get info for nearest point.") );
                }
                else
                {
                    // Display an info dialog
                    DisplayVectorFeatureInfo
                        ( 
                        theFindResults.front().mFeatureClassType, thePoint->mFeatureInfo 
                        );

                    // Free the point feature
                    GM_FreePointFeature( thePoint );
                    thePoint = NULL;
                }
            }
        }
        else
        {
            ::AfxMessageBox( _T("No features found near the click location.") );
        }
    }

    // Call the base class implementation
	CDialog::OnRButtonUp( nFlags, point );
} // end of function CGMDLL_TesterDlg::OnRButtonUp

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnSaveAsGeotiff
//
// DESCRIPTION:
//   Saves all currently loaded layers to a new GeoTIFF file.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnSaveAsGeotiff() 
{
    // Get the filename to save to
	CFileDialog theSaveDialog
        ( 
        FALSE,
        ".tif",
        NULL, 
        OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT, 
        "GeoTIFF (*.tif, *.tiff)|*.tif;*.tiff|All Files|*.*||"
        );
    if ( IDOK != theSaveDialog.DoModal() )
    {
        return;
    }
    
    // Save the file
    GM_Error_t32 theErr = GM_ExportRaster
        (
        theSaveDialog.GetPathName(),            // Name of new file to create
        GM_Export_GeoTIFF,                      // Format to export
        NULL,                                   // Layer to export or NULL for all
        NULL,                                   // World bounds to export or NULL for all
        2048,                                   // Width in pixels of new image
        2048,                                   // Height in pixels of new image
        GM_ExportFlags_GenWorldFile | GM_ExportFlags_GenPRJFile |
            GM_ExportFlags_GenTABFile |  GM_ExportFlags_Palette
        );
    if ( GM_Error_NotRegistered == theErr )
    {
        mMapWindow.ShowNotRegisteredMessage
            (
            _T("You cannot export GeoTIFF files without having a registration file.")
            );
    }
    else if ( GM_Error_None != theErr )
    {
        CString theErrStr;
        theErrStr.Format( "Error export GeoTIFF: %d", theErr );
        ::AfxMessageBox( theErrStr );
    }
    else
    {
        ::AfxMessageBox( "Succesfully exported GeoTIFF file" );
    }
} // end of function CGMDLL_TesterDlg::OnSaveAsGeotiff

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnSaveAsJpg
//
// DESCRIPTION:
//   Saves all currently loaded layers to a new JPG file.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnSaveAsJpg() 
{
    // Get the filename to save to
	CFileDialog theSaveDialog
        ( 
        FALSE,
        ".tif",
        NULL, 
        OFN_HIDEREADONLY | OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT, 
        "JPG (*.jpg)|*.jpg|All Files|*.*||"
        );
    if ( IDOK != theSaveDialog.DoModal() )
    {
        return;
    }
    
    // Save the file
    GM_Error_t32 theErr = GM_ExportRasterEx
        (
        theSaveDialog.GetPathName(),            // Name of new file to create
        GM_Export_JPG,                          // Format to export
        NULL,                                   // Layer list to export or NULL for all
        0,                                      // Number of layers in provided list (0 for all)
        NULL,                                   // World bounds to export or NULL for all
        2048,                                   // Width in pixels of new image
        2048,                                   // Height in pixels of new image
        GM_ExportFlags_GenWorldFile | GM_ExportFlags_GenPRJFile,
        "QUALITY=80"                            // Extra flags (specifying JPG export quality in this case)
        );
    if ( GM_Error_NotRegistered == theErr )
    {
        mMapWindow.ShowNotRegisteredMessage
            (
            _T("You cannot export JPG files without having a registration file.")
            );
    }
    else if ( GM_Error_None != theErr )
    {
        CString theErrStr;
        theErrStr.Format( "Error export JPG: %d", theErr );
        ::AfxMessageBox( theErrStr );
    }
    else
    {
        ::AfxMessageBox( "Succesfully exported JPG file" );
    }
} // end of function CGMDLL_TesterDlg::OnSaveAsJpg

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnSetProjToGeographic
//
// DESCRIPTION:
//   Sets the current view/export projection to geographic.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnSetProjToGeographic() 
{
    // Setup the projection object
    GM_Projection_t theProjDef;
    ::memset( &theProjDef, 0, sizeof theProjDef );
    theProjDef.mProjSys = GM_PRJ_GEO;
    theProjDef.mUnit = GM_PRJ_UNIT_ARC_DEGREES;

    // Get the datum from the current projection if there is one
    GM_Projection_t theCurProjDef;
    if ( GM_Error_None == GM_GetProjection( &theCurProjDef ) )
    {
        theProjDef.mDatum = theCurProjDef.mDatum;
    }
    else
    {
        // There isn't a current view projection (this means there is no data 
        // loaded), so just use WGS-84
        theProjDef.mDatum = GM_DATUM_WGS_84;
    }

    // Set the new view projection
    mMapWindow.SetViewProjection( &theProjDef );
} // end of function CGMDLL_TesterDlg::OnSetProjToGeographic

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnToggleAutoCrop
//
// DESCRIPTION:
//   Toggles auto-crop on and off for selected layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnToggleAutoCrop() 
{
    // Close each selected layer
    sint32 theSel = mLayerList.GetNextItem( -1, LVNI_SELECTED );
    if ( -1 == theSel )
    {
        ::AfxMessageBox( "You must select the layer(s) to remove the collars from." );
        return;
    }
    while ( -1 != theSel )
    {
        // Remove this layer
        GM_LayerHandle_t32 theLayer = (GM_LayerHandle_t32)mLayerList.GetItemData( theSel );

        // Get the current raster display options
        GM_RasterDisplayOptions_t theDispOpts;
        if ( GM_Error_None != mMapWindow.GetRasterDisplayOptions( theLayer, theDispOpts ) )
        {
            ::AfxMessageBox( _T("Unable to get raster display options for layer.") );
            break;
        }

        // Toggle the auto-crop setting
        theDispOpts.mAutoClipCollar = !theDispOpts.mAutoClipCollar;
        if ( GM_Error_None != mMapWindow.SetRasterDisplayOptions( theLayer, theDispOpts ) )
        {
            ::AfxMessageBox( _T("Unable to set raster display options for layer.") );
            break;
        }

        // Get the next selected item
        theSel = mLayerList.GetNextItem( theSel, LVNI_SELECTED );
    }
} // end of function CGMDLL_TesterDlg::OnToggleAutoCrop

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnZoomIn
//
// DESCRIPTION:
//   Zooms in by a factor of two.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnZoomIn() 
{
    mMapWindow.ZoomIn();
} // end of function CGMDLLTesterDlg::OnZoomIn

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnZoomOut
//
// DESCRIPTION:
//   Zooms out by a factor of two.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnZoomOut() 
{
    mMapWindow.ZoomOut();
} // end of function CGMDLLTesterDlg::OnZoomOut

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnZoomToAll
//
// DESCRIPTION:
//   Zooms to all loaded layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnZoomToAll() 
{
    mMapWindow.ZoomToAllLayers();
} // end of function CGMDLLTesterDlg::OnZoomToAll

//===================================================================
//
// METHOD:
//   CGMDLL_TesterDlg::OnZoomToSelLayer
//
// DESCRIPTION:
//   Zooms to the selected layers.
//
//===================================================================
void 
CGMDLL_TesterDlg::OnZoomToSelLayer() 
{
    // Build the bounds enclosing all selected layers
    bool isFirst = true;
    sint32 theSel = mLayerList.GetNextItem( -1, LVNI_SELECTED );
    GM_Rectangle_t theViewRect;
    if ( -1 == theSel )
    {
        ::AfxMessageBox( "You must select the layer(s) to zoom to." );
        return;
    }
    while ( -1 != theSel )
    {
        // Get the layer info
        GM_LayerHandle_t32 theLayer = 
            (GM_LayerHandle_t32)mLayerList.GetItemData( theSel );
        const GM_LayerInfo_t* theLayerInfo = GM_GetLayerInfo( theLayer );
        if ( NULL == theLayerInfo )
        {
            continue;
        }

        // Add the global bounds to this one
        if ( isFirst )
        {
            theViewRect = theLayerInfo->mGlobalRect;
            isFirst = false;
        }
        else
        {
            theViewRect.mMinX = 
                MIN_VAL( theViewRect.mMinX, theLayerInfo->mGlobalRect.mMinX );
            theViewRect.mMinY = 
                MIN_VAL( theViewRect.mMinY, theLayerInfo->mGlobalRect.mMinY );
            theViewRect.mMaxX = 
                MAX_VAL( theViewRect.mMaxX, theLayerInfo->mGlobalRect.mMaxX );
            theViewRect.mMaxY = 
                MAX_VAL( theViewRect.mMaxY, theLayerInfo->mGlobalRect.mMaxY );
        }

        // Get the next selected item
        theSel = mLayerList.GetNextItem( theSel, LVNI_SELECTED );
    }

    // Update the view rectangle
    mMapWindow.SetCurrentViewRect( theViewRect );
} // end of function CGMDLL_TesterDlg::OnZoomToSelLayer

// end of file GMDLL_TesterDlg.cpp
